/*==============================================================================*/
/*  Copyright (C) 2009-2016, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   GPIO 2bit Driver                                                 */
/*  EventID:   Insert Kernelinfo Test Point                                     */
/*  AppVer :   DT10 `Ver10.51                                                  */
/*  FileID :   34b                                                              */
/*  Version:   3.2                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define	DT_UINT unsigned int


/*==============================================================================*/
/*  Struct:  DT10_DATA                                                          */
/*  Desc:    Test Point Data Struct                                             */
/*==============================================================================*/
struct DT10_DATA {
	char tp_func;
	DT_UINT addr;
	DT_UINT dat;
	DT_UINT size;
	unsigned char value[256];
};

#define TP_BusOut		'2'
#define TP_MemoryOutput		'3'
#define TP_BusOutFast		'4'
#define TP_MemoryOutputFast		'5'
#define TP_EventTrigger		'6'
#define TP_EventTriggerFast		'7'
#define TP_EventTrigger32		'8'
#define TP_EventTrigger32Fast		'9'

/*==============================================================================*/
/*  Macro:  PROC_NAME                                                           */
/*  Desc:   Register Proc File name for Linux.                                  */
/*==============================================================================*/
#define DT10_DRV	"/proc/dt10proc"

static int dat_file = 0;
static int BusoutMode = 0;
/*==============================================================================*/
/*  Macro:  BusOutModeSet                                                       */
/*  Desc:   0 => open file 1 => writing 2 => close file                         */
/*==============================================================================*/
#define BusOutModeSet(mode) BusoutMode = mode

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut(DT_UINT addr, DT_UINT dat)
{
	struct DT10_DATA dt10_data;
	
	if (BusoutMode == 0) {
		dat_file = open(DT10_DRV, O_RDWR);
		if (dat_file <= 0) {
			printf("[DT10] %s open error \n", DT10_DRV);
			return;
		}
		printf("[DT10] %s open ok \n", DT10_DRV);
		BusOutModeSet(1);
	}

	dt10_data.tp_func = TP_BusOut;
	dt10_data.addr = addr;
	dt10_data.dat = dat;
	
	write(dat_file, &dt10_data, sizeof(dt10_data));
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput(DT_UINT addr, DT_UINT dat, void *value, DT_UINT size)
{
	struct DT10_DATA dt10_data;

	if (BusoutMode == 0) {
		dat_file = open(DT10_DRV, O_RDWR);
		if (dat_file <= 0) {
			printf("[DT10] %s open error \n", DT10_DRV);
			return;
		}
		BusOutModeSet(1);
	}
	dt10_data.tp_func = TP_MemoryOutput;
	dt10_data.addr = addr;
	dt10_data.dat = dat;
	dt10_data.size = size;
	memcpy(dt10_data.value, value, size);
	
	write(dat_file, &dt10_data, sizeof(dt10_data));
}

/*==============================================================================*/
/*  Func:   _TP_BusOutFast                                                      */
/*  Desc:   Called by Fast Test Point                                           */
/*==============================================================================*/
void _TP_BusOutFast( DT_UINT step, DT_UINT bit_num )
{
	struct DT10_DATA dt10_data;
	
	if (BusoutMode == 0) {
		dat_file = open(DT10_DRV, O_RDWR);
		if (dat_file <= 0) {
			printf("[DT10] %s open error \n", DT10_DRV);
			return;
		}
		printf("[DT10] %s open ok \n", DT10_DRV);
		BusOutModeSet(1);
	}

	dt10_data.tp_func = TP_BusOutFast;
	dt10_data.addr = step;
	dt10_data.dat = bit_num;
	
	write(dat_file, &dt10_data, sizeof(dt10_data));
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutputFast                                                */
/*  Desc:   Called by Variable Fast Test Point                                  */
/*==============================================================================*/
void _TP_MemoryOutputFast( DT_UINT step, DT_UINT bit_num, void *value, DT_UINT size )
{
	struct DT10_DATA dt10_data;

	if (BusoutMode == 0) {
		dat_file = open(DT10_DRV, O_RDWR);
		if (dat_file <= 0) {
			printf("[DT10] %s open error \n", DT10_DRV);
			return;
		}
		BusOutModeSet(1);
	}
	dt10_data.tp_func = TP_MemoryOutputFast;
	dt10_data.addr = step;
	dt10_data.dat = bit_num;
	dt10_data.size = size;
	memcpy(dt10_data.value, value, size);
	
	write(dat_file, &dt10_data, sizeof(dt10_data));
}

/*==============================================================================*/

